=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-07-20 05:11:18 +0000
+++ bzrlib/errors.py	2006-07-20 11:03:38 +0000
@@ -1029,7 +1029,7 @@
 class MalformedFooter(BadBundle): pass
 
 
-class NoSmartServer(BzrNewError):
+class NoSmartServer(NotBranchError):
     """No smart server available at %(url)s"""
 
     def __init__(self, url):

=== added file 'bzrlib/remote.py'
--- bzrlib/remote.py	1970-01-01 00:00:00 +0000
+++ bzrlib/remote.py	2006-07-20 11:03:38 +0000
@@ -0,0 +1,64 @@
+# Copyright (C) 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+from bzrlib import branch, errors
+from bzrlib.bzrdir import BzrDir, BzrDirFormat
+from bzrlib.branch import Branch
+from bzrlib.trace import mutter
+from bzrlib.transport.smart import SmartTransport
+
+
+class RemoteBzrDirFormat(BzrDirFormat):
+    """Format representing bzrdirs accessed via a smart server"""
+    
+    def probe_transport(self, transport):
+        mutter("%r probe for bzrdir in %r" % (self, transport))
+        if isinstance(transport, SmartTransport):
+            return self
+        else:
+            raise errors.NoSmartServer(transport.base)
+
+    def _open(self, transport):
+        return RemoteBzrDir(transport)
+
+
+class RemoteBzrDir(BzrDir):
+    """Control directory on a remote server, accessed by HPSS."""
+
+    def __init__(self, transport):
+        BzrDir.__init__(self, transport, RemoteBzrDirFormat)
+        self.client = transport.get_smart_client()
+
+    def open_branch(self):
+        return RemoteBranch(self, self.client)
+
+
+class RemoteBranch(branch.BzrBranch5):
+
+    def __init__(self, my_bzrdir, smart_client):
+        self.bzrdir = my_bzrdir
+        self.client = smart_client
+
+
+# when first loaded, register this format.
+#
+# TODO: Actually this needs to be done earlier; we can hold off on loading
+# this code until it's needed though.
+
+# We can't use register_control_format because it adds it at a lower priority
+# than the existing branches, whereas this should take priority.
+BzrDirFormat._control_formats.insert(0, RemoteBzrDirFormat())

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-07-20 06:13:57 +0000
+++ bzrlib/tests/__init__.py	2006-07-20 11:03:38 +0000
@@ -1284,6 +1284,7 @@
                    'bzrlib.tests.test_plugins',
                    'bzrlib.tests.test_progress',
                    'bzrlib.tests.test_reconcile',
+                   'bzrlib.tests.test_remote',
                    'bzrlib.tests.test_repository',
                    'bzrlib.tests.test_revision',
                    'bzrlib.tests.test_revisionnamespaces',

=== added file 'bzrlib/tests/test_remote.py'
--- bzrlib/tests/test_remote.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_remote.py	2006-07-20 11:03:38 +0000
@@ -0,0 +1,60 @@
+# Copyright (C) 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for remote bzrdir/branch/repo/etc
+
+These are proxy objects which act on remote objects by sending messages
+through a smart client.  The proxies are to be created when attempting to open
+the object given a transport that supports smartserver rpc operations. 
+"""
+
+from bzrlib import bzrdir, remote, tests
+from bzrlib.transport import smart
+from bzrlib.bzrdir import BzrDir, BzrDirFormat
+from bzrlib.remote import RemoteBzrDir, RemoteBzrDirFormat
+from bzrlib.branch import Branch
+
+class BasicRemoteObjectTests(tests.TestCaseInTempDir):
+
+    def setUp(self):
+        tests.TestCaseInTempDir.setUp(self)
+        self.server = smart.SmartTCPServer_for_testing()
+        self.server.setUp()
+        self.addCleanup(self.server.tearDown)
+        self.transport = smart.SmartTCPTransport(self.server.get_url())
+        self.client = self.transport.get_smart_client()
+        # make a branch that can be opened over the smart transport
+        BzrDir.create_standalone_workingtree('.')
+
+    def test_create_remote_bzrdir(self):
+        b = remote.RemoteBzrDir(self.transport)
+        self.assertIsInstance(b, BzrDir)
+
+    def test_open_remote_branch(self):
+        # create a standalone branch in the working directory
+        b = remote.RemoteBzrDir(self.transport)
+        branch = b.open_branch()
+
+    def test_find_correct_format(self):
+        """Should open a RemoteBzrDir over a SmartTransport"""
+        fmt = BzrDirFormat.find_format(self.transport)
+        ## self.assert_(RemoteBzrDirFormat in BzrDirFormat._control_formats)
+        self.assertIsInstance(fmt, remote.RemoteBzrDirFormat)
+
+    def test_open_detected_smart_format(self):
+        fmt = BzrDirFormat.find_format(self.transport)
+        d = fmt.open(self.transport)
+        self.assertIsInstance(d, BzrDir)

=== modified file 'bzrlib/tests/test_smart_transport.py'
--- bzrlib/tests/test_smart_transport.py	2006-07-20 06:13:57 +0000
+++ bzrlib/tests/test_smart_transport.py	2006-07-20 11:03:38 +0000
@@ -43,18 +43,28 @@
 
     def setUp(self):
         super(TCPClientTests, self).setUp()
+        # We're allowed to set  the transport class here, so that we don't use
+        # the default or a parameterized class, but rather use the
+        # TestCaseWithTransport infrastructure to set up a smart server and
+        # transport.
+        #
+        # At the momen the SmartServer transport is only readonly, but
+        # TestCaseWithTransport tries to make sure the transport is writable.
         self.transport_readonly_server = smart.SmartTCPServer_for_testing
 
     def test_plausible_url(self):
         self.assert_(self.get_readonly_url().startswith('bzr://'))
 
+    def test_probe_transport(self):
+        t = self.get_readonly_transport()
+        self.assertIsInstance(t, smart.SmartTransport)
+
     def test_get_client_from_transport(self):
         t = self.get_readonly_transport()
         client = t.get_smart_client()
         self.assertIsInstance(client, smart.SmartStreamClient)
 
 
-
 class BasicSmartTests(tests.TestCase):
     
     def test_smart_query_version(self):

=== modified file 'bzrlib/transport/smart.py'
--- bzrlib/transport/smart.py	2006-07-20 06:13:57 +0000
+++ bzrlib/transport/smart.py	2006-07-20 11:03:38 +0000
@@ -125,8 +125,6 @@
 # it's quite useful to be able to deal with Transports representing locations
 # without actually opening it.
 #
-# TODO: Better name for this.
-#
 # TODO: Split the actual smart server from the ssh encoding of it.
 #
 # TODO: Perhaps support file-level readwrite operations over the transport

